@@ -185,8 +185,8 @@ class SciPyFST:
             outStates.append(curentState)
             outSignals.append(self.getOutSignal(curentState, inSignal, -1))
             curentState = self.getNextState(curentState, inSignal)#, curentState)
+        outStates.append(curentState)
         if self.isMoore():
-            outStates.append(curentState)
             outSignals.append(self.getOutSignal(curentState, inSignal, -1))
 
         return outSignals, outStates
@@ -275,8 +275,16 @@ class SciPyFST:
         wave = "{ \"signal\": [" + waveCLK + waveRST + waveCMD + waveSTT + waveOUT + "],\"config\":{\"hscale\":" + str(hscale) + "}}"
         return wave
 
-    def toDot(self):
+    def toDot(self, **kwargs):
         """
+        nameGV = 'fst'\n
+        rankdirGV = 'LR'\n
+        colorOfUnreachableStates = 'aqua'\n
+        highlightStates = []\n
+        highlightStatesColor = 'lightblue'\n
+        highlightPath = []\n
+        highlightPathColor = 'red3'\n
+        \n
         !!! DRAFT !!!
         Output example:\n
         digraph fst {\n
@@ -295,32 +303,75 @@ class SciPyFST:
         }
         """
 
+        nameGV = kwargs.pop('nameGV', 'fst')
+        rankdirGV = kwargs.pop('rankdirGV', 'LR')
+        colorOfUnreachableStates = kwargs.pop('colorOfUnreachableStates', None)
+        highlightStates = kwargs.pop('highlightStates', [])
+        highlightStatesColor = kwargs.pop('highlightStatesColor', 'lightblue')
+        highlightPath = kwargs.pop('highlightPath', None)
+        highlightPathColor = kwargs.pop('highlightPathColor', 'red3')
+
         ifNotInDict = '-'
-        outString = "digraph fst {\n\trankdir=LR;\n\tnode [shape=point]; start;\n\tnode [shape=circle];"
-        if self.isMoore():
-            outString += " \"{initState}\" [label=\"{initState}/{outSignal}\"];".format(
-                initState = str(self.initState), outSignal = self.getOutSignal(self.initState, None, ifNotInDict))
+        unreachableStates = self.getUnreachableStates() if colorOfUnreachableStates is not None else []
+        if highlightPath is not None:
+            hlPathStates = self.playFST(highlightPath)[1]
+            hlPathTransition = list(zip(hlPathStates, highlightPath, hlPathStates[1:]))
         else:
-            outString += " \"{initState}\" [label=\"{initState}\"];".format(initState = str(self.initState))
+            hlPathStates = []
+            hlPathTransition = []
+
+        # Dot header
+        outString = "digraph {} {{\n\trankdir={};\n\tnode [shape=point]; start;".format(nameGV, rankdirGV)
+        # InitState
+        outString += "\n\tnode [shape=circle];"
+        nodeStyle = "style=filled, fillcolor={}, ".format(highlightStatesColor) if self.initState in highlightStates else ""
+        nodeStyle2 = "color={hlc}, fontcolor={hlc}, style=bold, ".format(hlc=highlightPathColor) if self.initState in hlPathStates else ""
+        if self.isMoore():
+            outString += "\n\t\"{initState}\" [{style}{style2}label=\"{initState}/{outSignal}\"];".format(
+                initState = str(self.initState),
+                outSignal = self.getOutSignal(self.initState, None, ifNotInDict),
+                style = nodeStyle,
+                style2 = nodeStyle2)
+        else:
+            outString += "\n\t\"{initState}\" [{style}{style2}label=\"{initState}\"];".format(
+                initState = str(self.initState),
+                style = nodeStyle,
+                style2 = nodeStyle2)
         outString += "\n\tstart -> \"{initState}\" [label=start];\n\tnode [shape=circle];".format(initState = str(self.initState))
+        # all state
         for state in self.states:
             if state != self.initState:
-                if self.isMoore():
-                    outString += "\n\t\"{state}\" [label=\"{state}/{outSignal}\"];".format(
-                        state = state, outSignal = self.getOutSignal(state, None, ifNotInDict))
+                if state in highlightStates:
+                    nodeStyle = "style=filled, fillcolor={}, ".format(highlightStatesColor)
+                elif state in unreachableStates:
+                    nodeStyle = "style=filled, fillcolor={}, ".format(colorOfUnreachableStates)
                 else:
-                    outString += "\n\t\"{state}\" [label=\"{state}\"];".format(state = state)
+                    nodeStyle = ""
+                nodeStyle2 = "color={hlc}, fontcolor={hlc}, style=bold, ".format(hlc=highlightPathColor) if state in hlPathStates else ""
+                if self.isMoore():
+                    outString += "\n\t\"{state}\" [{style}{style2}label=\"{state}/{outSignal}\"];".format(
+                        state = state,
+                        style = nodeStyle,
+                        style2 = nodeStyle2,
+                        outSignal = self.getOutSignal(state, None, ifNotInDict))
+                else:
+                    outString += "\n\t\"{state}\" [{style}{style2}label=\"{state}\"];".format(
+                        state = state,
+                        style = nodeStyle,
+                        style2 = nodeStyle2)
         outString += "\n\tnode [style=filled, fillcolor=hotpink];"
+        # transition
         for (state, inSignal, nextState) in self.transitionFunction:
+            pathStyle = "color={hlc}, fontcolor={hlc}, style=bold, ".format(hlc=highlightPathColor) if (state, inSignal, nextState) in hlPathTransition else ""
             if nextState is None:
                 nextState = ifNotInDict
             if self.isMoore():
-                outString += "\n\t\"{state}\" -> \"{nextState}\" [label={inSignal}];".format(
-                    state = str(state), nextState = str(nextState), inSignal = str(inSignal))
+                outString += "\n\t\"{state}\" -> \"{nextState}\" [{style}label={inSignal}];".format(
+                    state = str(state), nextState = str(nextState), inSignal = str(inSignal), style = pathStyle)
             else:
-                outString += "\n\t\"{state}\" -> \"{nextState}\" [label=\"{inSignal}/{outSignal}\"];".format(
+                outString += "\n\t\"{state}\" -> \"{nextState}\" [{style}label=\"{inSignal}/{outSignal}\"];".format(
                     state = str(state), nextState = str(nextState), inSignal = str(inSignal),
-                    outSignal = self.getOutSignal(state, inSignal, ifNotInDict))
+                    outSignal = self.getOutSignal(state, inSignal, ifNotInDict), style = pathStyle)
         outString += "\n}"
         return outString
 
@@ -426,7 +477,7 @@ class SciPyFST:
         recGetNext(self.initState, dict(), [])
         return listOfInSignalsList
 
-    def isContains(self, fst):
+    def _isContains(self, fst:'SciPyFST'):
         listOfInSignalsList = fst.getTestSignal()
         selfType = 1 if self.isMoore() else 0
         fstType = 1 if fst.isMoore() else 0
@@ -435,5 +486,42 @@ class SciPyFST:
                 return False
         return True
 
-    def isSimilar(self, fst):
+    def isContains(self, fst:'SciPyFST'):
+        selfType = 1 if self.isMoore() else 0
+        fstType = 1 if fst.isMoore() else 0
+        def recGetNext(curentState, visitedStates:dict, inSignals:list):
+            if visitedStates.get(curentState) is None:
+                visitedStates[curentState] = 1
+                for inSignal in fst.inAlphabet:
+                    copyOfInSignals = deepcopy(inSignals)
+                    copyOfInSignals.append(inSignal)
+                    nextCurentState = fst.getNextState(curentState, inSignal)
+                    if nextCurentState is None:
+                        return self.playFST(copyOfInSignals)[0][selfType:] == fst.playFST(copyOfInSignals)[0][fstType:]
+                    else:
+                        if not recGetNext(nextCurentState, deepcopy(visitedStates), copyOfInSignals):
+                            return False
+            else:
+                return self.playFST(inSignals)[0][selfType:] == fst.playFST(inSignals)[0][fstType:]
+            return True
+        return recGetNext(fst.initState, dict(), [])
+
+    def isSimilar(self, fst:'SciPyFST'):
         return self.isContains(fst) and fst.isContains(self)
+
+    def getUnreachableStates(self):
+        unreachableStates = dict.fromkeys(self.states)
+        def recGetNext(curentState, visitedStates:dict):
+            unreachableStates.pop(curentState, None)
+            if visitedStates.get(curentState) is None:
+                visitedStates[curentState] = 1
+                for inSignal in self.inAlphabet:
+                    nextCurentState = self.getNextState(curentState, inSignal)
+                    if nextCurentState is None:
+                        return
+                    else:
+                        recGetNext(nextCurentState, deepcopy(visitedStates))
+            else:
+                return
+        recGetNext(self.initState, dict())
+        return sorted(unreachableStates)
